## Load package cdfquantreg
library("cdfquantreg")

##********************************************
## Section 2.2: Random number generator
##********************************************
## Random number generator
set.seed(0310)
(x <- rq(5, mu = 0.5, sigma = 1, "arcsinh", "cauchy"))

## Density
dq(x, mu = 0.5, sigma = 1, "arcsinh", "cauchy") 

## Cumulative density
(cpv <- pq(x, mu = 0.5, sigma = 1, "arcsinh", "cauchy"))

## Quantile
qq(cpv, mu = 0.5, sigma = 1, "arcsinh", "cauchy")


##********************************************
## Section 3.1: IPCC
##********************************************

data("cdfqrExampleData", package = "cdfquantreg")

## Prepare for the IPCC data;
dataipcc <- subset(IPCC, mid == 1 & high == 0) 
## Fit beta regression model
library("betareg")
betafit <- betareg(probm ~ valence | valence, data = dataipcc)
summary(betafit)
## Call:
##  betareg(formula = probm ~ valence | valence, data = dataipcc)
## 
## Standardized weighted residuals 2:
##   Min      1Q  Median      3Q     Max 
## -6.3577 -0.4799  0.1135  0.5037  4.8358 
## 
## Coefficients (mean model with logit link):
##              Estimate Std. Error z value Pr(>|z|)    
##  (Intercept)  0.51164    0.04052  12.626  < 2e-16 ***
##   valence     0.14961    0.05248   2.851  0.00436 ** 
##   
##   Phi coefficients (precision model with log link):
##               Estimate Std. Error  z value Pr(>|z|)    
##   (Intercept)  0.97728    0.04816    20.29   <2e-16 ***
##   valence      0.60259    0.06990     8.62   <2e-16 ***
##   ---
##   Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1 
## 
## Type of estimator: ML (maximum likelihood)
## Log-likelihood: 264.7 on 4 Df
## Pseudo R-squared: 0.000607
## Number of iterations: 16 (BFGS) + 1 (Fisher scoring) 

## Fit cdfquantreg model
fit <- cdfquantreg(probm ~ valence | valence, fd = "t2", sd = "t2", data = dataipcc)
summary(fit)

## Plot fit
plot(fit) 
 
## Plot fit using the estimated mu and sigma for each condition
ind <- which(dataipcc$valence == 0) 	
coefs <- coef(fit); fitv <- fitted(fit)
dneg <- dq(fitv[ind], coefs[1], exp(coefs[3]), "t2", "t2")
dpos <- dq(fitv[-ind], sum(coefs[1:2]), exp(sum(coefs[3:4])), "t2", "t2")
neg <- data.frame(x = fitv[ind], d = dneg)[order(fitv[ind]), ]
pos <- data.frame(x = fitv[-ind], d = dpos)[order(fitv[-ind]), ]

library("MASS")
par(mfrow = c(1, 2))
truehist(dataipcc$probm[ind], col = "white", ylim = c(0, 3), 
         main = "Negative Valence", xlab = "Best Estimate")
lines(neg$x, neg$d, lty = 1, lwd = 2)
truehist(dataipcc$probm[-ind], col = "white", ylim = c(0, 3), 
         main = "Positive Valence", xlab = "Best Estimate")
lines(pos$x, pos$d, lty = 1, lwd = 2)

##********************************************
## Compare with quantreg
xx <- seq(0.005, 0.995, .005)
cdf_neg <- pq(xx, coefs[1], exp(coefs[3]), "t2", "t2"); 
cdf_pos <- pq(xx, sum(coefs[1:2]), exp(sum(coefs[3:4])), "t2", "t2")

fitqtot <- quantreg::rq(probm ~ valence, tau = xx, data = dataipcc)

par(mfrow = c(1, 2), mar = c(4, 3, 3, 1))
plot(xx, cdf_neg, type = "l", lty = 1, 
     main = "Negative Valence", xlab = "Probability", ylab = "")
title(ylab = "CDF", line = 2, cex.lab = 1.2)
lines(coef(fitqtot)[1, ], xx, lty = 2)
plot(xx, cdf_pos, type = "l", lty = 1, 
     main = "Positive Valence", xlab = "Probability", ylab = "")
title(ylab = "CDF", line = 2, cex.lab = 1.2)
lines(colSums(coef(fitqtot)[1:2, ]), xx, lty = 2)
legend("topleft", c("Cdf-quantreg", "Quantreg"), 
       lty = 1:2, bty = "n")


##********************************************
## Check influence

infs <- influence(fit, plot = TRUE)	


##********************************************
## Section 3.2: Continuous predictor
##********************************************
mod0 <- cdfquantreg(bestprob ~ cfprob | 1, fd = "logit", sd = "logistic", data = IPCCAUS)
summary(mod0)
mod1 <- update(mod0, . ~ + cfprob * gender)

anova(mod0, mod1)

summary(mod1)

## Plot for Figure 6
x <- IPCCAUS$cfprob
newx <- seq(-sd(x), sd(x), .01) + mean(x)

newdat <- data.frame(cfprob = newx , gender = 1, `cfprob:gender` = newx)
pred_m <- predict(mod1, newdat)

newdat[, 2:3] <- newdat[, 2:3] * 0 
pred_fm <- predict(mod1, newdat)

plot(newx, pred_fm, type = "l", lty = 1, ylim = c(0.5, 0.75), 
    xlab = "Context-Free Rating", ylab = "Best Estimate")
lines(newx, pred_m, lty = 2)
legend("topleft", c("Females", "Males"), lty = c(1, 2))


##********************************************
## Section 3.2: Copulas
##********************************************
library("lmtest") 
library("copula") # "copula" package version 0.999-16 is required
## One-stage procedure
## Provide the density, cdf, and quantile t2-t2 functions:
dqt2 <- function(x, mu, sigma) dq(x, mu, sigma, "t2", "t2")
qqt2 <- function(x, mu, sigma) qq(x, mu, sigma, "t2", "t2")
pqt2 <- function(x, mu, sigma) pq(x, mu, sigma, "t2", "t2")

## Form a trivariate t-copula applied to Q4-6 (positive valence) using
## the T2-T2 distribution in the marginals.
tCop1 <- tCopula(c(0.6, 0.3, 0.3), dim = 3, dispstr = "un", df.fixed = FALSE)
margin1 <- list(mu = 1.5, sigma = 1) # parameters for the marginal distributions
MvdPV1 <- mvdc(copula = tCop1, margins = c("qt2", "qt2", "qt2"), 
               paramMargins = list(margin1, margin1, margin1)) 

## Fit to Q4-6 (positive valence) data:
start <- c(1.5, 1, 1.5, 1, 1.5, 1, 0.6, 0.3, 0.3,4)
copfitPV1 <- fitMvdc(data = as.matrix(IPCC_Wide[ , 1:3]), mvdc = MvdPV1, 
                     start = start, 
                     optim.control = list(trace = TRUE, maxit = 2000))
logLik(copfitPV1)
## "log Lik." 336.3636 (df=10)

## Alternative way to form copula with a "sphericity" model that
## assumes equal correlations among the three variables:
tCop2 <- tCopula(0.5, dim = 3, dispstr = "ex", df.fixed = FALSE)
MvdPV2 <- mvdc(copula = tCop2, 
               margins = c("qt2", "qt2", "qt2"), 
               paramMargins = list(margin1, margin1, margin1))

start <- c(1.5, 1, 1.5, 1, 1.5, 1, 0.5, 4)
copfitPV2 <- fitMvdc(
  as.matrix(IPCC_Wide[, 1:3]),
  MvdPV2,
  start = start,
  optim.control = list(trace = TRUE, maxit = 2000)
)

## The log-likelihoods are similar for both kinds of model:
lrtest(copfitPV1, copfitPV2)

AIC(copfitPV1, copfitPV2)

## Two-stage model
## Here are the cdfquantile models.
fit4 <- cdfquantreg(Q4 ~ 1 | 1, fd = "t2", sd = "t2", data = IPCC_Wide)
fit5 <- cdfquantreg(Q5 ~ 1 | 1, fd = "t2", sd = "t2", data = IPCC_Wide)
fit6 <- cdfquantreg(Q6 ~ 1 | 1, fd = "t2", sd = "t2", data = IPCC_Wide)

## Get the uniform-marginal pseudo-observations for Q4-6:
udat <- cbind(pqt2(IPCC_Wide[, 1], coef(fit4)[1], exp(coef(fit4)[2])),
              pqt2(IPCC_Wide[, 2], coef(fit5)[1], exp(coef(fit5)[2])),
              pqt2(IPCC_Wide[, 3], coef(fit6)[1], exp(coef(fit6)[2])))
## Now to fit the copula:
copfitudatPV <- fitCopula(tCop2, udat, start = c(0.5, 2.5))
## Compare the log-likelihood of the marginal structure 
## with the one-stage model:
loglikMvdc(c(coef(fit4)[1], exp(coef(fit4)[2]),
             coef(fit5)[1], exp(coef(fit5)[2]),
             coef(fit6)[1], exp(coef(fit6)[2]),
             coef(copfitudatPV)), 
             as.matrix(IPCC_Wide)[, 1:3], MvdPV2)

## Table 3
## One-stage model output:
## The m1, m2 and m3 coefficient outputs correspond to the mu/sigma 
## 4, 5, and 6 in Table 3 in the paper.
## The rho and df estimates correspond to the rho and psi estimate in
## Table 3.
summary(copfitPV2)

## Two-stage model output:
## The coefficient estimates for rho and psi.
summary(copfitudatPV)

## The coefficient estimates and se for mu4, 5 and 6 can be obtained via  
## the output of the fitted cdfquantile model.
summary(fit4)
summary(fit5)
summary(fit6)

## The coefficient estimates and se for sigma4, 5, and 6 can be obtained 
## by back-transforming the dispersion model parameters and using the vcov
## functions.
c(exp(coef(fit4)[2]), 
  sqrt((exp(vcov(fit4)[2, 2]) - 1) * exp(2 * coef(fit4)[2] + vcov(fit4)[2, 2])))
c(exp(coef(fit5)[2]), 
  sqrt((exp(vcov(fit5)[2, 2]) - 1) * exp(2 * coef(fit5)[2] + vcov(fit5)[2, 2])))
c(exp(coef(fit6)[2]), 
  sqrt((exp(vcov(fit6)[2, 2]) - 1) * exp(2 * coef(fit6)[2] + vcov(fit6)[2, 2])))


##********************************************
## Section 3.3. Ambiguity-conflict data
##********************************************
Ambdata$prob <- scaleTR(Ambdata$value,high = 70, low = 0, N = 1570, scale = 0.5) 
options(digits = 3)

##** Fit T2-T2 distribution*************
fit1_t2t2 <- cdfquantreg(prob ~ 1 | 1, fd = "t2", sd = "t2", data = Ambdata)
fit2_t2t2 <- update(fit1_t2t2, . ~ + scenario | 1)
fit3_t2t2 <- update(fit2_t2t2, . ~ . | + scenario)
anova(fit1_t2t2, fit2_t2t2, fit3_t2t2)

## Dispersion coefficients
fit3_t2t2$coefficients$dispersion

##** Fit Logit-Logistic distribution *************
fit1_loglog <- cdfquantreg(prob ~ 1 | 1, fd = "logit", sd = "logistic", data = Ambdata)
fit2_loglog <- update(fit1_loglog, . ~ + scenario | 1)
fit3_loglog <- update(fit2_loglog, . ~ . | + scenario)
anova(fit1_loglog, fit2_loglog, fit3_loglog)

## Dispersion coefficients
fit3_loglog$coefficients$dispersion

## Fit ArcSinh-Cauchy distribution*************
fit1_ac <- cdfquantreg(prob ~ 1 | 1, fd = "arcsinh", sd = "t2", data = Ambdata)
fit2_ac <- update(fit1_ac, . ~ + scenario | 1)
fit3_ac <- update(fit2_ac, . ~ . | + scenario)
anova(fit1_ac, fit2_ac, fit3_ac)

## Dispersion coefficients
fit3_ac$coefficients$dispersion


##********************************************
## Section 3.5 Simulation
##********************************************
## Over view the three distribution shapes
s <- seq(0.0001, 0.9999, length.out = 10000)
d1 <- dq(s, mu = 0, sigma = 0.377033, "logit", "cauchy")
d2 <- dq(s, mu = 0, sigma = 0.910239, "logit", "cauchy")
d3 <- dq(s, mu = 0, sigma = 2.19751, "logit", "cauchy")

par(mfrow = c(1, 3), mar = c(2, 2, 2, 1), oma = c(3, 3, 0, 0))
plot(d1, type = "l"); plot(d2, type = "l"); plot(d3, type = "l")
mtext("Case Number", side = 1, outer = TRUE, line = 1)
mtext("Density", side = 2, outer = TRUE, line = 1)

## Generate random numbers from the target distribution
s1 <- rq(10000, mu = 0, sigma = 0.377033, "logit", "cauchy")
s2 <- rq(10000, mu = 0, sigma = 0.910239, "logit", "cauchy")
s3 <- rq(10000, mu = 0, sigma = 2.19751, "logit", "cauchy")

## Summary of the simulated quantiles
simout <- matrix(data = c(rep(0, 12)), nrow = 3, ncol = 3)
rownames(simout) <- c("s1", "s2", "s3")
colnames(simout) <- c("p25", "p50", "p75")
simout[1, ] <- quantile(s1, c(.25, .5, .75))
simout[2, ] <- quantile(s2, c(.25, .5, .75))
simout[3, ] <- quantile(s3, c(.25, .5, .75))
simout

## Compare the simulated quantiles and the true quantiles
s1 <- qq(c(.25, .5, .75), mu = 0, sigma = 0.377033, "logit", "cauchy")
s2 <- qq(c(.25, .5, .75), mu = 0, sigma = 0.910239, "logit", "cauchy")
s3 <- qq(c(.25, .5, .75), mu = 0, sigma = 2.19751, "logit", "cauchy")
rbind(s1, s2, s3)


##********************************************
## Figure 1
##********************************************
settings <- data.frame(
  name = rep(c("Logit-Logistic (Logit-logistic)", "Bimodal (T2-chauchy)", "Finite-Tailed (Arcsinh-arcsinh)", "Trimodal (Cauchit-T2)"), each = 6),
  fd = rep(c("logit", "t2", "arcsinh", "cauchit"), each = 6),
  sd = rep(c("logistic", "cauchy", "arcsinh", "t2"), each = 6),
  mu = c(0, 0, 0, -1.4, -0.7, 1.4, 0, 0, 0, -1.2, -0.7, 1.2, 0, 0, 0, -1.2, -0.7, 1.2, 0, 0, 0, -1.2, -0.7, 1.2),
  sigma = c(0.3, 1, 1.5, 0.8, 0.8, 0.8, 0.4, 1, 1.8, 1.2, 1.2, 1.2, 0.4, 1, 1.8, 1.2, 1.2, 1.2, 0.4, 1, 1.8, 0.8, 0.8, 0.8),
  stringsAsFactors = FALSE
)

pdf("Figure1.pdf", width = 11, height = 7)
par(mfrow = c(2, 2), mar = c(2.1, 4.1, 4.1, 10.1), xpd = TRUE)
for (scenario in unique(settings$name)) {
  pars <- subset(settings, name == scenario)
  x <- seq(0, 1, length.out = 1000)[-c(1, 1000)]
  densities <- do.call("cbind", lapply(seq_len(nrow(pars)),
    function(i) dq(x, pars$mu[i], pars$sigma[i], pars$fd[i], pars$sd[i])))
  is.na(densities[densities > 3.5]) <- TRUE
  matplot(x, densities, main = scenario, ylim = c(0, 3.5), xlab = "x", ylab = "density", type = "l", lwd = 2)
  legend("right", inset = c(-0.51, 0),
    legend = as.expression(lapply(seq_len(nrow(pars)),
      function(i) bquote(mu~'='~.(pars$mu[i])~', '~sigma~'='~.(pars$sigma[i])))),
    lty = 1:6, col = 1:6, lwd = 2, bty = "n")
}
dev.off()
